@@ -1,4 +1,6 @@
 import openai
+import time
+import json
 
 from uuid import uuid4
 
@@ -16,7 +18,11 @@ class Thread:
         self.id = str(uuid4())
 
     def __len__(self):
-        """ Returns the length of the Thread, by message count """
+        """ Returns the length of the Thread, by message count
+
+        Returns:
+            int: Length of thread by message count
+        """
 
         return len(self.thread)
 
@@ -31,7 +37,12 @@ class Thread:
 
     def serialize(self):
         """ Returns a serializable, JSON-friendly dict with all of the
-        thread's data. Can be restored to a new Thread object later. """
+        thread's data. Can be restored to a new Thread object later.
+
+        Returns:
+            dict: A JSON-safe representation of this thread, to be restored
+                later using thread.restore()
+        """
 
         return {
             "__quickgpt-thread__": {
@@ -42,7 +53,14 @@ class Thread:
 
     def restore(self, obj):
         """ Restores a serialized Thread object,
-        provided by thread.serialize() """
+        provided by thread.serialize()
+
+        Args:
+            obj (dict): A dict returned from thread.serialize()
+
+        Returns:
+            None
+        """
 
         thread_dict = obj["__quickgpt-thread__"]
         self.id = thread_dict["id"]
@@ -58,7 +76,7 @@ class Thread:
                 self.thread.remove(message)
 
     def feed(self, *messages, insert=None):
-        """ Appends a new message to the Thread feed. """
+        """ Appends a new message to the end of the Thread feed. """
 
         try:
             # Check if the first argument is a list, and then make it the parent
@@ -94,7 +112,9 @@ class Thread:
                 self.thread.append(msg)
 
     def insert(self, index, *messages):
-        """ Inserts messages at a given index in the Thread """
+        """ Inserts messages at a given index in the Thread
+
+        Unfinished method?"""
 
         self.feed(
             insert=True,
@@ -103,24 +123,55 @@ class Thread:
 
     @property
     def messages(self):
-        """ Returns a JSON-safe list of all messages in this thread """
+        """ Returns a JSON-safe list of all messages in this thread
+
+        Returns:
+            list: JSON-safe list of all messages
+        """
 
         return [ msg.obj for msg in self.thread ]
 
     def run(self, *append_messages, feed=True):
-        """ Executes the current Thread and get a response. If `feed` is
+        """ Executes the current Thread and get a response. If ``feed`` is
         True, it will automatically save the response to the Thread.
 
-        You can provide *messages that will only apply to this run(), and
-        won't be stored in the Thread. """
+        You can provide *append_messages that will only apply to this run(),
+        and won't be permanently stored in the Thread.
+
+        Returns:
+            Response: The resulting Response object from OpenAI
+        """
 
         messages = self.messages
         append_messages = [ msg.obj for msg in append_messages ]
 
-        response_obj = openai.ChatCompletion.create(
-            model=self.model,
-            messages=messages + append_messages
-        )
+        for i in range(self.quickgpt.retry_count):
+            try:
+                response_obj = openai.ChatCompletion.create(
+                    model=self.model,
+                    messages=messages + append_messages
+                )
+
+                break
+            except openai.error.RateLimitError as e:
+                if self.quickgpt.retry_count == i - 1:
+                    print("Failed after %s tries" % self.quickgpt.retry_count)
+                    raise e
+
+                print("OpenAI returned RateLimitError, trying again after 10 sec...")
+                time.sleep(10)
+            except openai.error.InvalidRequestError as e:
+                print(
+                    json.dumps({
+                        "messages": messages + append_messages,
+                        "model": self.model
+                    }, indent=2)
+                )
+
+                print("Please see failed request encoded as JSON above.")
+
+                raise e
+
 
         response = Response(response_obj)
 
@@ -130,7 +181,14 @@ class Thread:
         return response
 
     def moderate(self, prompt):
-        """ Validate a prompt to ensure it's safe for OpenAI's policies """
+        """ Validate a prompt to ensure it's safe for OpenAI's policies
+
+        Args:
+            prompt (str): The user's query to validate
+
+        Returns:
+            bool: True if it's flagged as a violation, False if it's safe
+        """
 
         response = openai.Moderation.create(
             input=prompt
